//=============================================================================
// GUIPage. Base for all GUI pages.
//=============================================================================
class GUIPage expands GUIBase;

#EXEC TEXTURE IMPORT FILE="Display99.bmp" NAME="Display99" GROUP="GUI" MIPS=off FLAGS=2 TEXFLAGS=0
#EXEC AUDIO IMPORT FILE="GraphSlide.wav" NAME="GraphSlide" GROUP="GUI"

var GUIComponent Components[32];
var GUIComponent MouseOver,PressedComponent,TypingComponent;
var int NumComponents,MouseX,MouseY;
var bool bIsFocus,bMouseIsClicked,bPressedBoarder,bBoarderMIcon;
var GUIPage NextPage;
var() texture BackgorundTex;
var() string MenuTitle;
var() bool bAllowMoving;
var(Sounds) Sound MenuOpenSound;
var float BrightnessTime,MaxXMouse,MaxYMouse,PressedOffset[2];
struct ShortCoord
{
	var() int X,Y,XL,YL;
};
var() struct BcgCoords
{
	var() ShortCoord UL,L,DL,D,DR,R,UR,U,B,T,BP;
} BcgCrd; // Background coordinates
var string ForcedTypeStr;
var UTPlayer Owner;
var GUIPageManager GUIManager;

// U = Up
// L = Left
// R = Right
// D = Down
// B = Background coords
// T = Title text
// Size/Pos in 0-1 so it will be ok for all res.
var() float StartPosX,StartPosY,SizeX,SizeY;

// Spawn needed buttons here.
simulated function InitilizePage()
{
	bIsFocus = True;
	MenuSound(MenuOpenSound);
}

// Render everything here. (Cmd for if having text input)
simulated function RenderPage( Canvas C )
{
	local float XC,YC;
	
	if( BrightnessTime<1 )
	{
		C.DrawColor.R = BrightnessTime*255;
		C.DrawColor.B = BrightnessTime*255;
		C.DrawColor.G = BrightnessTime*255;
	}
	else if( !bIsFocus )
	{
		C.DrawColor.R = 128;
		C.DrawColor.B = 128;
		C.DrawColor.G = 128;
	}
	else
	{
		C.DrawColor.R = 255;
		C.DrawColor.B = 255;
		C.DrawColor.G = 255;
	}
	if( ForcedTypeStr!="" )
	{
		if( !C.Viewport.Console.IsInState('Typing') )
			InputFromPlayer("");
		else if( Left(C.Viewport.Console.TypedStr,Len(ForcedTypeStr))!=ForcedTypeStr )
			C.Viewport.Console.TypedStr = ForcedTypeStr;
	}
	C.Style = 3;
	XC = C.ClipX;
	YC = C.ClipY;
	MaxXMouse = C.ClipX;
	MaxYMouse = C.ClipY;
	C.ClipX = XC*SizeX;
	C.ClipY = YC*SizeY;
	C.OrgX = int(XC*StartPosX);
	C.OrgY = int(YC*StartPosY);
	RenderPageBase(C,C.ClipX,C.ClipY);
	RenderPageContent(C,XC,YC);
	C.ClipX = XC;
	C.ClipY = YC;
	C.OrgX = 0;
	C.OrgY = 0;
	C.DrawColor.R = 255;
	C.DrawColor.B = 255;
	C.DrawColor.G = 255;
}
// Render the boarder and title
simulated function RenderPageBase( Canvas C, int SizeX, int SizeY )
{
	local int X,Y,OX;
	local float XL,YL;
	
	// Up Left
	C.SetPos(0,0);
	C.DrawTile(BackgorundTex,BcgCrd.UL.X,BcgCrd.UL.Y,BcgCrd.UL.XL,BcgCrd.UL.YL,BcgCrd.UL.X,BcgCrd.UL.Y);
	// Left
	C.SetPos(0,BcgCrd.UL.Y);
	C.DrawTile(BackgorundTex,BcgCrd.L.X,SizeY-BcgCrd.DL.Y-BcgCrd.UL.Y,BcgCrd.L.XL,BcgCrd.L.YL,BcgCrd.L.X,BcgCrd.L.Y);
	// Down Left
	C.SetPos(0,SizeY-BcgCrd.DL.Y);
	C.DrawTile(BackgorundTex,BcgCrd.DL.X,BcgCrd.DL.Y,BcgCrd.DL.XL,BcgCrd.DL.YL,BcgCrd.DL.X,BcgCrd.DL.Y);
	// Down
	C.SetPos(BcgCrd.DL.X,SizeY-BcgCrd.DL.Y);
	C.DrawTile(BackgorundTex,SizeX-BcgCrd.DL.X-BcgCrd.DR.X,BcgCrd.D.Y,BcgCrd.D.XL,BcgCrd.D.YL,BcgCrd.D.X,BcgCrd.D.Y);
	// Down Right
	C.SetPos(SizeX-BcgCrd.DR.X,SizeY-BcgCrd.DL.Y);
	C.DrawTile(BackgorundTex,BcgCrd.DR.X,BcgCrd.DR.Y,BcgCrd.DR.XL,BcgCrd.DR.YL,BcgCrd.DR.X,BcgCrd.DR.Y);
	// Right
	C.SetPos(SizeX-BcgCrd.UR.X,BcgCrd.UR.Y);
	C.DrawTile(BackgorundTex,BcgCrd.R.X,SizeY-BcgCrd.DR.Y-BcgCrd.UR.Y,BcgCrd.R.XL,BcgCrd.R.YL,BcgCrd.R.X,BcgCrd.R.Y);
	// Up Right
	C.SetPos(SizeX-BcgCrd.UR.X,0);
	C.DrawTile(BackgorundTex,BcgCrd.UR.X,BcgCrd.UR.Y,BcgCrd.UR.XL,BcgCrd.UR.YL,BcgCrd.UR.X,BcgCrd.UR.Y);
	// Up
	C.SetPos(BcgCrd.UL.X,0);
	C.DrawTile(BackgorundTex,SizeX-BcgCrd.UL.X-BcgCrd.UR.X,BcgCrd.U.Y,BcgCrd.U.XL,BcgCrd.U.YL,BcgCrd.U.X,BcgCrd.U.Y);
	// Boarder
	C.SetPos(BcgCrd.UL.X,BcgCrd.UL.Y);
	C.DrawTile(BackgorundTex,SizeX-BcgCrd.UL.X-BcgCrd.DR.X,SizeY-BcgCrd.UL.Y-BcgCrd.DR.Y,BcgCrd.B.XL,BcgCrd.B.YL,BcgCrd.B.X,BcgCrd.B.Y);
	// Title
	if( MenuTitle=="" )
		Return; // Skip this if no title.
	OX = C.OrgX;
	C.OrgX+=BcgCrd.T.XL;
	C.SetPos(0,BcgCrd.T.YL);
	C.Font = Font'WhiteFont';
	X = C.ClipX;
	Y = C.ClipY;
	C.ClipX = SizeX-BcgCrd.T.X-BcgCrd.T.XL;
	C.ClipY = SizeY-BcgCrd.T.Y-BcgCrd.T.YL;
	C.TextSize(MenuTitle,XL,YL);
	C.CurX = int(C.ClipX/2-XL/2);
	C.DrawTextClipped(MenuTitle,False);
	C.ClipX = X;
	C.ClipY = Y;
	C.OrgX = OX;
}
// Render the content/handel mouse.
simulated function RenderPageContent( Canvas C, float XL, float YL )
{
	local int i;
	local float MX,MY;
	local GUIComponent HighestComponent;
	local bool bPressing;
	
	C.Style = 1;
	MX = (MouseX-C.OrgX)/XL;
	MY = (MouseY-C.OrgY)/YL;
	For( i=0; i<NumComponents; i++ )
	{
		if( TypingComponent==Components[i] )
			Components[i].RenderComponent(C,XL,YL,"");
		else Components[i].RenderComponent(C,XL,YL);
		if( Components[i].MouseOn(MX,MY) )
			HighestComponent = Components[i];
	}
	bPressing = (C.ViewPort.Actor.bFire!=0 || C.ViewPort.Actor.bAltFire!=0);
	if( bPressing && bPressedBoarder )
	{
		StartPosX-=PressedOffset[0]-MX;
		StartPosY-=PressedOffset[1]-MY;
		if( StartPosX<0 )
			StartPosX = 0;
		else if( (StartPosX+SizeX)>1 )
			StartPosX = 1-SizeX;
		if( StartPosY<0 )
			StartPosY = 0;
		else if( (StartPosY+SizeY)>1 )
			StartPosY = 1-SizeY;
		bBoarderMIcon = True;
		Return;
	}
	if( HighestComponent!=MouseOver )
	{
		if( MouseOver!=None )
			MouseOver.BecameUnfocus(bPressing);
		MouseOver = HighestComponent;
		if( (bPressing && PressedComponent==MouseOver && MouseOver!=None) || (!bPressing && MouseOver!=None) )
			MouseOver.BecameFocus(bPressing);
	}
	if( bMouseIsClicked!=bPressing )
	{
		bMouseIsClicked = bPressing;
		if( bPressing )
		{
			PressedComponent = MouseOver;
			if( MouseOver!=None )
				MouseOver.MouseClick(C.ViewPort.Actor.bAltFire!=0,False);
			else if( IsBoarder(MouseX,MouseY,XL,YL) )
			{
				bPressedBoarder = True;
				PressedOffset[0] = MX;
				PressedOffset[1] = MY;
				bBoarderMIcon = True;
			}
		}
		else if( bPressedBoarder )
			bPressedBoarder = False;
		else if( MouseOver!=None )
		{
			if( !MouseOver.MouseClick(C.ViewPort.Actor.bAltFire!=0,True) )
				InterfaceAction(MouseOver);
		}
		else if( IsBoarder(MouseX,MouseY,XL,YL) )
			bBoarderMIcon = True;
	}
	else if( IsBoarder(MouseX,MouseY,XL,YL) )
			bBoarderMIcon = True;
}
// Add a component, return none if list is full.
simulated function GUIComponent AddContent( Class<GUIComponent> GClass )
{
	if( GClass==None || NumComponents==ArrayCount(Components) )
		Return None;
	Components[NumComponents] = New(Self,'GUIPart')GClass;
	Components[NumComponents].GUIPage = Self;
	Components[NumComponents].Initilize();
	Components[NumComponents].ID = NumComponents;
	NumComponents++;
	Return Components[NumComponents-1];
}
// Add a normal button.
simulated function AddButton( float PosX, float PosY, float SizeX, float SizeY, string Text, optional string InfoTxt )
{
	local GUIButton GB;
	
	GB = GUIButton(AddContent(Class'GUIButton'));
	if( GB==None ) Return;
	GB.SizeX = SizeX;
	GB.SizeY = SizeY;
	GB.PosX = PosX;
	GB.PosY = PosY;
	GB.ButtonText = Text;
	GB.ButtonDetails = InfoTxt;
}
// Make fade-in effect.
simulated function Tick( float Delta )
{
	BrightnessTime+=Delta*2;
	if( BrightnessTime>=1 )
	{
		BrightnessTime = 1;
		Disable('Tick');
	}
}
// Play menu sound on client.
simulated function MenuSound( sound S )
{
	if( Owner!=None )
	{
		if( Owner.ViewTarget!=None )
			Owner.ViewTarget.PlaySound(S,,1,,10000);
		else Owner.PlaySound(S,,1,,10000);
	}
}
// Check if mouse is over the drag/drop boarder
simulated function bool IsBoarder( int MX, int MY, int WinX, int WinY )
{
	local float EX,EY;
	
	if( !bAllowMoving ) Return False;
	MX-=(StartPosX*WinX);
	MY-=(StartPosY*WinY);
	EY = BcgCrd.BP.YL+BcgCrd.BP.Y;
	EX = (SizeX*WinX)-BcgCrd.BP.X;
	Return (MX>=BcgCrd.BP.XL && MX<=EX && MY>=BcgCrd.BP.YL && MY<=EY);
}
// A component was pressed.
simulated function InterfaceAction( GUIComponent Other );

// Start GUI typing
simulated function StartGUITyping( GUIComponent ExpectingComponent, bool bSendForServer )
{
	if( bSendForServer )
		ForcedTypeStr = "GUICInput ";
	else ForcedTypeStr = "GUIInput ";
	Owner.Player.Console.TypedStr = ForcedTypeStr;
	Owner.Player.Console.GoToState('Typing');
	TypingComponent = ExpectingComponent;
}
// Must call this after typing is done.
simulated function EndGUITyping()
{
	TypingComponent = None;
	ForcedTypeStr = "";
	Owner.Player.Console.GoToState('');
}

// Got input from player
simulated function InputFromPlayer( string Cmd );

// Got input from client (Serverside)
static function ClientInput( UTPlayer Other, string Cmd );

// Request server to send data to client
simulated function GetDataFromServer( optional string S )
{
	Owner.ServerGetGUIData(Class,S);
}

// Server, send some data for the client.
// Call on Receiver.ClientReceiveGUIData( Default.Class, <data here> ) to process the info for client GUI
Static function SendClientData( UTPlayer Receiver, optional string S );

// Data received from server:
simulated function ReceivedData( string S );

// Forward the data to correct GUI page
simulated function ForwardSInput( Class<GUIPage> GuiClass, string S )
{
	if( GuiClass==Class )
		ReceivedData(S);
	else if( NextPage!=None )
		NextPage.ForwardSInput(GuiClass,S);
}

defaultproperties
{
				BackgorundTex=Texture'AQS.GUI.Display99'
				MenuTitle="GUI Page"
				bAllowMoving=True
				BcgCrd=(UL=(X=40,Y=28),L=(X=40,Y=1,YL=28),DL=(X=40,Y=29,YL=99),D=(X=1,Y=29,XL=40,YL=99),DR=(X=38,Y=29,XL=90,YL=99),R=(X=38,Y=1,XL=90,YL=30),UR=(X=38,Y=28,XL=90),U=(X=1,Y=28,XL=40),B=(X=1,Y=1,XL=64,YL=64),t=(X=40,Y=26,XL=40,YL=7),BP=(X=38,Y=22))
				StartPosX=0.200000
				StartPosY=0.250000
				SizeX=0.800000
				SizeY=0.700000
}
